/*
 * Input event loop functionality.
 *
 * (c) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#ifdef __cplusplus

#include <l4/re/event>
#include <l4/sys/capability>
#include <l4/sys/icu>

#include <pthread.h>
#include <pthread-l4.h>

/* POSIX-thread-compatible handler function type. */

typedef void (*Event_handler)(L4Re::Event_buffer::Event &, void *);

/* Input event loop abstraction. */

class Input_event_loop
{
  /* Notification interrupt capability. */

  L4::Cap<L4::Irq> _irq;

  /* External handler function. */

  Event_handler _handler;

  /* Private data for the handler. */

  void *_priv;

  /* Event buffer and notification interrupt capability. */

  L4Re::Event_buffer _event_buffer;

  /* Thread properties. */

  int _priority;
  pthread_t _pthread;

public:
  /* Initialise the event loop with a notification interrupt, thread priority,
     event handler function and private data, and an event buffer. */

  explicit Input_event_loop(L4::Cap<L4::Irq> irq,
                            Event_handler handler, void *priv,
                            L4Re::Event_buffer event_buffer,
                            int priority=0x20)
  : _irq(irq), _handler(handler), _priv(priv), _event_buffer(event_buffer),
    _priority(priority)
  {
  }

  /* Static main loop function for event-handling thread. */

  static void *event_loop(void *data);

  /* Event handler method. */

  void handle();

  /* Initiation function. */

  void start();
};

#endif
